﻿const fs = require('fs')
const spawn = require('child_process').spawn;
const date = require('date-and-time'); 

const Timer = {

    
    /** @param {object} ws - Web Server Object */
    CRONWEB: function (ws) {

        const path = __dirname + '/cronweb';
        const config = require(path + '/config.json');
        var launchers = config.Launchers;
        var Interval = config.Interval;
        var taches = require(path + '/taches.json').Taches; 
        var scripts = [];
        var ptr = null;
        var tick = null;
        var jours = ['lundi', 'mardi', 'mercredi', 'jeudi', 'vendredi', 'samedi', 'dimanche'];
        var statut = { "working": false, "scripts": scripts, "taches": taches}

        date.locale('fr');

        //charge les scipts
        fs.readdirSync(path + '/scripts/').forEach(file => {
            for (var i = 0; i < launchers.length; i++) {
                if (file.lastIndexOf(launchers[i].ext) > 0 ) { scripts.push(file); }
            };          
        });

        //demarre le timer
        function start() {
            if (tick) { return; }
            tick = setInterval(timer, Interval * 1000);
            statut.working = true;
            sendstatut();
        }

        //stop le timer
        function stop () {
            if (tick) {
                clearInterval(tick); 
                statut.working = false;
                tick = null;
                sendstatut();
            }
        }
        
        //timer toute les 'config.Interval' secondes
        function timer () {
            var a = statut.taches;            
            var now = new Date();
            var ptrnow = date.format(now, 'ddddHHmm');
            for (var i = 0; i < a.length; i++) {
                if (a[i].jour == date.format(now, 'dddd') &&
                    a[i].heure == parseInt(date.format(now, 'HH')) &&
                    a[i].minute == parseInt(date.format(now, 'mm')) &&
                    ptr != ptrnow) {
                    exec(a[i].script);
                }
            }
            ptr = ptrnow;
        }

        //execute le script 's'
        function exec(s) {
            var l = null;
            var d = null;
            for (var i = 0; i < launchers.length; i++) {
                if (s.lastIndexOf(launchers[i].ext) > 0)
                {                   
                    d = launchers[i].using == '' ? '' : path + '/scripts/' + s;
                    l = launchers[i].using == '' ? path + '/scipts/' + s : launchers[i].using
                }
            }; 
            if (l != null) {
                var p = spawn(l, [d]);
                p.stdout.on('data', function (data) { sendmessage(data); });
                p.stderr.on('error', function (err) { sendmessage(err); });
            }
        }

        //sauvegarde les actions
        function save() {        
            fs.writeFile(path + '/taches.json', JSON.stringify({ "Taches": statut.taches }), 'utf-8', function (err) {
                sendstatut();
            });
        }

        //Ajoute l'action 'a'
        function add(a) {                    
            a.tag  = jours.indexOf(a.jour) + ('0' + a.heure).slice(-2) + ('0' + a.minute).slice(-2) + a.script;
            if (JSON.stringify(statut.taches).indexOf(a.tag) == -1) {              
                statut.taches.push(a);
                save();
            }
        }

        //supprime l'action 'a'
        function del(a) {
            var x = [];
            for (i = 0; i < statut.taches.length; i++) {
                if (statut.taches[i].tag != a.tag) { x.push(statut.taches[i]); }
            }
            statut.taches = x;
            save();
        }


        //broadcast les messages sur le soket
        function sendmessage(m) {
            ws.io.sockets.emit('timer.cronweb.message', date.format(new Date(), 'dddd DD MMM à HH:mm') + ' : ' + m);
        }

        //broadcast les données sur le socket
        function sendstatut() {
            ws.io.sockets.emit('timer.cronweb.statut', statut);
        }

        //routes GET
        ws.app.get('/timer.cronweb', (req, res) => {
            res.render("timer.cronweb.html");
        });

             
        //routes Socket.IO
        ws.io.sockets.on('connection', (client) => {
            client.emit('timer.cronweb.statut', statut);

            client.on('timer.cronweb.get.statut', () => {
                client.emit('timer.cronweb.statut', statut );
            });

          
            client.on('timer.cronweb.add.tache', (data) => {
                add(data.tache);
            });

            client.on('timer.cronweb.del.tache', (data) => {
                del(data.tache);
            });

            client.on('timer.cronweb.run.script', (data) => {
                exec(data.script);
            });

            client.on('timer.cronweb.start', () => {              
                start(10); 
            });

            client.on('timer.cronweb.stop', () => {
                stop();
            });

        }); 


        this.Start = () => { start(); }
        this.Stop = () => { stop(); }

        console.log('CronWEB Started. Tick Every ' + Interval + ' Seconde(s)');
        
    }

};
module.exports = Timer;

